@@ -20,6 +20,7 @@ import (
 
 	cluster "github.com/envoyproxy/go-control-plane/envoy/config/cluster/v3"
 	core "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
+	dfpcluster "github.com/envoyproxy/go-control-plane/envoy/extensions/clusters/dynamic_forward_proxy/v3"
 	tlsv3 "github.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/v3"
 	http "github.com/envoyproxy/go-control-plane/envoy/extensions/upstreams/http/v3"
 	"google.golang.org/protobuf/types/known/structpb"
@@ -189,7 +190,6 @@ func constructUpstreamTLS(opts *buildClusterOpts, tls *networking.ClientTLSSetti
 		CommonTlsContext: defaultUpstreamCommonTLSContext(),
 		Sni:              tls.Sni,
 	}
-
 	setAutoSniAndAutoSanValidation(c, tls)
 
 	// Use subject alt names specified in service entry if TLS settings does not have subject alt names.
@@ -283,10 +283,33 @@ func setAutoSniAndAutoSanValidation(mc *clusterWrapper, tls *networking.ClientTL
 	if len(tls.Sni) == 0 {
 		setAutoSni = true
 	}
-	if setAutoSni && len(tls.SubjectAltNames) == 0 && !tls.GetInsecureSkipVerify().GetValue() {
+	// For DFP clusters, we only set auto_san_validation if insecure_skip_verify is false.
+	if (setAutoSni && len(tls.SubjectAltNames) == 0 || mc.isDFPCluster) && !tls.GetInsecureSkipVerify().GetValue() {
 		setAutoSanValidation = true
 	}
 
+	// For DFP clusters, we need to allow_insecure_cluster_options if insecure_skip_verify is true.
+	if mc.isDFPCluster && tls.GetInsecureSkipVerify().GetValue() {
+		clusterType, ok := mc.cluster.ClusterDiscoveryType.(*cluster.Cluster_ClusterType)
+		if !ok {
+			log.Warnf("failed to set allow_insecure_cluster_options for DFP cluster %s: invalid cluster type", mc.cluster.Name)
+			return
+		}
+		customClusterType := clusterType.ClusterType
+		if customClusterType == nil {
+			log.Warnf("failed to set allow_insecure_cluster_options for DFP cluster %s: missing custom cluster type", mc.cluster.Name)
+			return
+		}
+		dfpCluster := &dfpcluster.ClusterConfig{}
+		if err := customClusterType.TypedConfig.UnmarshalTo(dfpCluster); err != nil {
+			log.Warnf("failed to unmarshal dynamic forward proxy config: %v", err)
+			return
+		}
+		dfpCluster.AllowInsecureClusterOptions = true
+		customClusterType.TypedConfig = protoconv.MessageToAny(dfpCluster)
+		log.Debugf("set allow_insecure_cluster_options for DFP cluster %s", mc.cluster.Name)
+	}
+
 	if setAutoSni || setAutoSanValidation {
 		if mc.httpProtocolOptions == nil {
 			mc.httpProtocolOptions = &http.HttpProtocolOptions{}
